-- Deploy camper:campsite_type_pet_cost to pg
-- requires: roles
-- requires: schema_camper
-- requires: campsite_type
-- requires: user_profile

begin;

set search_path to camper, public;

create table campsite_type_pet_cost (
	campsite_type_id integer primary key references campsite_type,
	cost_per_night integer not null constraint cost_not_negative check(cost_per_night >= 0)
);

grant select on table campsite_type_pet_cost to guest;
grant select on table campsite_type_pet_cost to employee;
grant select, insert, update, delete on table campsite_type_pet_cost to admin;

alter table campsite_type_pet_cost enable row level security;

create policy guest_ok
on campsite_type_pet_cost
for select
using (true)
;

create policy insert_to_company
on campsite_type_pet_cost
for insert
with check (
	exists (select 1 from campsite_type join user_profile using (company_id) where campsite_type.campsite_type_id = campsite_type_pet_cost.campsite_type_id)
)
;

create policy update_company
on campsite_type_pet_cost
for update
using (
	exists (select 1 from campsite_type join user_profile using (company_id) where campsite_type.campsite_type_id = campsite_type_pet_cost.campsite_type_id)
)
;

create policy delete_from_company
on campsite_type_pet_cost
for delete
using (
	exists (select 1 from campsite_type join user_profile using (company_id) where campsite_type.campsite_type_id = campsite_type_pet_cost.campsite_type_id)
)
;

commit;
